package com.bocloud.rms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.rms.entity.ServiceBill;
import com.bocloud.rms.intf.BillReportDownloadService;
import com.bocloud.rms.intf.ServiceBillService;
import com.bocloud.rms.repository.ServiceBillRepository;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Base64;
import java.util.List;
import java.util.Map;
import java.util.concurrent.*;

/**
 * @author admin
 */
@Service("billReportDownloadService")
public class BillReportDownloadServiceImpl implements BillReportDownloadService {
    private static final Logger logger = LoggerFactory.getLogger(BillReportDownloadServiceImpl.class);
    private final ServiceBillService serviceBillService;
    private final ServiceBillRepository billRepository;
    private static final ThreadLocal<Integer> ROW = new ThreadLocal<>();

    public BillReportDownloadServiceImpl(ServiceBillService serviceBillService, ServiceBillRepository billRepository) {
        this.serviceBillService = serviceBillService;
        this.billRepository = billRepository;
    }

    @Override
    public BocloudResult<Map<String, Object>> billReportDownload(Pager pager, RequestContext context, String category) {
        SXSSFWorkbook workbook = new SXSSFWorkbook();
        try {
            SXSSFSheet sheet = workbook.createSheet("sheet");
            sheet.setRandomAccessWindowSize(-1);
            // 样式一，标题数据蓝色填充
            CellStyle style = workbook.createCellStyle();
            style.setFillForegroundColor(IndexedColors.ROYAL_BLUE.getIndex());
            style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            style.setBorderBottom(BorderStyle.THIN); // 下边框
            style.setBorderLeft(BorderStyle.THIN);// 左边框
            style.setBorderTop(BorderStyle.THIN);// 上边框
            style.setBorderRight(BorderStyle.THIN);// 右边框
            style.setAlignment(HorizontalAlignment.CENTER);
            // 样式二，内容样式
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderTop(BorderStyle.THIN);// 上边框
            cellStyle.setBorderBottom(BorderStyle.THIN); // 下边框
            cellStyle.setBorderLeft(BorderStyle.THIN);// 左边框
            cellStyle.setBorderRight(BorderStyle.THIN);// 右边框
            cellStyle.setAlignment(HorizontalAlignment.CENTER);// 水平对齐方式
            cellStyle.setVerticalAlignment(VerticalAlignment.TOP);// 垂直对齐方式
            // 标题写入
            createBillReport(sheet, style);
            // 读写数据
            int total = billRepository.count(Lists.newArrayList(), null, null, null);
            // 每个线程处理的数量
            int numbers = 20;
            if (total < numbers) {
                excelExporter(sheet, cellStyle, 1, new Pager(1, numbers, pager.getParams(), pager.getSorter()), context);
            } else {
                int pages = total / numbers;
                int lastNumers = total % numbers;
                ThreadPoolExecutor mainExecutor = new ThreadPoolExecutor(10, total, 5, TimeUnit.MINUTES, new ArrayBlockingQueue<>(total), new ThreadFactoryBuilder().setNameFormat("billReportDownloadExecutor").build());
                CompletionService<Boolean> completionService = new ExecutorCompletionService<>(mainExecutor);
                int rowNumber = 1;
                for (int i = 1; i <= pages; i++) {
                    int page = i;
                    int finalRowNumber = rowNumber;
                    completionService.submit(() -> excelExporter(sheet, cellStyle, finalRowNumber,
                            new Pager(page, numbers, pager.getParams(), pager.getSorter()), context));
                    rowNumber += numbers;
                }
                // 获取结果
                for (int j = 1; j <= pages; j++) {
                    Boolean successful = completionService.take().get();
                    if (successful) {
                        // 收到写入完成提示
                        continue;
                    }
                }
                mainExecutor.shutdown();
                mainExecutor.awaitTermination(5, TimeUnit.MINUTES);
                // 整页数据的异步查询写入完成，处理最后一页不足整页的数据
                if (0 != lastNumers) {
                    excelExporter(sheet, cellStyle, pages * numbers + 1,
                            new Pager(pages + 1, numbers, pager.getParams(), pager.getSorter()), context);
                }
            }
            String fileName;
            switch (category) {
                case "operation":
                    fileName = "运营账单统计报表";
                    break;
                case "tenant":
                    fileName = "租户账单统计报表";
                    break;
                default:
                    return new BocloudResult<>(false, "传参格式有误请检查");
            }
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            workbook.write(byteArrayOutputStream);
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName + System.currentTimeMillis() + ".xlsx");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult<>(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("账单报表下载失败", e);
            return new BocloudResult<>(false, "账单报表下载失败");
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                logger.error("error message：", e);
            }
        }
    }

    public Boolean excelExporter(SXSSFSheet sheet, CellStyle cellStyle, int rowNumber, Pager pager,
                                 RequestContext context) {
        int number = rowNumber;
        try {
            BocloudResult<GridBean<ServiceBill>> result = serviceBillService.list(pager, context);
            if (result.isFailed()) {
                logger.error("获取账单数据失败,{}", result.getMessage());
                return false;
            }
            List<JSONObject> data = JSONObject.parseObject(JSON.toJSONString(result.getData())).getJSONArray("rows")
                    .toJavaList(JSONObject.class);
            for (JSONObject resource : data) {
                ROW.set(number += 1);
                // 创建数据初始化行
                SXSSFRow sheetRow = sheet.createRow(ROW.get());
                sheetRow.createCell(0).setCellValue(resource.getString("month"));
                sheet.getRow(ROW.get()).getCell(0).setCellStyle(cellStyle);
                sheetRow.createCell(1).setCellValue(resource.getString("serviceName"));
                sheet.getRow(ROW.get()).getCell(1).setCellStyle(cellStyle);
                sheetRow.createCell(2).setCellValue(resource.getString("sn"));
                sheet.getRow(ROW.get()).getCell(2).setCellStyle(cellStyle);
                String status = resource.getString("status");
                switch (status) {
                    case "unpay":
                        status = "未支付";
                        break;
                    case "paid":
                        status = "已支付";
                        break;
                    case "paying":
                        status = "支付中";
                        break;
                    case "canceld":
                        status = "已取消";
                        break;
                    default:
                        break;
                }
                sheetRow.createCell(3).setCellValue(status);
                sheet.getRow(ROW.get()).getCell(3).setCellStyle(cellStyle);
                String chargeMode = resource.getString("chargeMode");
                if ("PostCharge".equalsIgnoreCase(chargeMode)) {
                    chargeMode = "后付费";
                } else if ("PreCharge".equalsIgnoreCase(chargeMode)) {
                    chargeMode = "预付费";
                }
                sheetRow.createCell(4).setCellValue(chargeMode);
                sheet.getRow(ROW.get()).getCell(4).setCellStyle(cellStyle);
                sheetRow.createCell(5).setCellValue(resource.getString("orderSn"));
                sheet.getRow(ROW.get()).getCell(5).setCellStyle(cellStyle);
                sheetRow.createCell(6).setCellValue(String.valueOf(resource.getDouble("money")));
                sheet.getRow(ROW.get()).getCell(6).setCellStyle(cellStyle);
                sheetRow.createCell(7).setCellValue(String.valueOf(resource.getString("gmtCreate")));
                sheet.getRow(ROW.get()).getCell(7).setCellStyle(cellStyle);
                sheetRow.createCell(8).setCellValue(String.valueOf(resource.getString("tenantName")));
                sheet.getRow(ROW.get()).getCell(8).setCellStyle(cellStyle);
            }
            return true;
        } catch (Exception e) {
            logger.error("账单报表【" + ROW.get() + "行至" + (ROW.get() + 20) + "行】写入失败", e);
            return false;
        }
    }

    public void createBillReport(SXSSFSheet sheet, CellStyle style) {
        // 模板标题处理
        String[] titles = {"账期", "资源类型", "流水号", "账单状态", "计费类型", "订单号", "账单金额(元)", "创建时间", "租户名称"};
        int number = -1;
        SXSSFRow row = sheet.createRow(0);
        for (String title : titles) {
            number++;
            row.createCell(number).setCellValue(title);
            sheet.getRow(0).getCell(number).setCellStyle(style);
            sheet.setColumnWidth(number, 20 * 256);
            sheet.addMergedRegion(new CellRangeAddress(0, 1, number, number));
        }
    }
}
